try:
    import urllib.parse
except ImportError:
    import urlparse

import requests

from conftest import TEST_API_KEY, EXPECTED_INTERACT_URL, TEST_MSISDN


def parse_qs(url):
    """Return URL parameters as a dictionary of name/value pairs."""
    return dict(urlparse.parse_qsl(urlparse.urlsplit(url).query))


def create_mock_rbm_response(req):
    """(Eventually) create a response we would expect from RBM given a request."""
    return req


def test_notify_suggested_reply_provides_proper_default_parameters(direqt, requests_mock):
    """SDK provides expected default parameters fetch endpoint.
    :param direqt.DireqtClient direqt: default Direqt test client
    """

    def verify_request(request):
        """:type request: requests.models.PreparedRequest"""
        qs = parse_qs(request.url)

        assert request.body is None
        assert qs['key'] == TEST_API_KEY
        assert qs['format'] == 'RBMV1.0'
        assert 'adUnit' not in qs
        assert 'targeting' not in qs

        return True

    requests_mock.post(EXPECTED_INTERACT_URL, json={}, additional_matcher=verify_request)

    result = direqt.notify_suggested_reply(TEST_MSISDN, '@direqt-fake-postback-data')

    assert requests_mock.call_count == 1
    assert result is True



import json
try:
    import urllib.parse
except ImportError:
    import urlparse

import requests

from conftest import TEST_API_KEY, EXPECTED_FETCH_URL, TEST_MSISDN, EXPECTED_AGENT_MESSAGE_URL, \
    EXPECTED_FILE_MESSAGE_URL, TEST_MSISDN_MD5, TEST_SUBSCRIBER_ID


def parse_qs(url):
    """Return URL parameters as a dictionary of name/value pairs."""
    return dict(urlparse.parse_qsl(urlparse.urlsplit(url).query))


def create_mock_rbm_response(req):
    """(Eventually) create a response we would expect from RBM given a request."""
    return req


def test_fetch_provides_proper_default_parameters(direqt, requests_mock):
    """SDK provides expected default parameters fetch endpoint.
    :param direqt.DireqtClient direqt: default Direqt test client
    """

    def verify_request(request):
        """:type request: requests.models.PreparedRequest"""
        qs = parse_qs(request.url)

        assert request.body is None
        assert qs['key'] == TEST_API_KEY
        assert qs['format'] == 'RBMV1.0'
        assert qs['adUnit'] == 'test-ad-unit'
        assert qs['targeting'] == '{}'

        return True

    requests_mock.post(EXPECTED_FETCH_URL, json={}, additional_matcher=verify_request)

    result = direqt.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert result is False


def test_fetch_handles_empty_response(direqt, requests_mock):
    """handles empty fetch response.
    :param requests_mock.Mocker requests_mock
    :param direqt.DireqtClient direqt
    """
    requests_mock.post(EXPECTED_FETCH_URL, json={})

    result = direqt.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert result is False


def test_fetch_simple_result(direqt, requests_mock):
    """handles a simple text response.

    Verify that when the Direqt server returns a simple text payload, that payload
    is sent along to RBM as expected.

    :param requests_mock.Mocker requests_mock
    :param direqt.DireqtClient direqt
    """
    rbm_message = {
        'contentMessage': {
            'text': 'Hello, world!'
        }
    }

    direqt_response = {
        'payload': json.dumps(rbm_message)
    }

    def verify_rbm_request(request):
        """:type request: requests.models.PreparedRequest"""
        assert request.body == json.dumps(rbm_message)
        return True

    requests_mock.post(EXPECTED_FETCH_URL, json=direqt_response)  # direqt/fetch
    requests_mock.post(EXPECTED_AGENT_MESSAGE_URL, json=create_mock_rbm_response(rbm_message),
                       additional_matcher=verify_rbm_request)

    result = direqt.fetch('test-ad-unit', TEST_MSISDN)

    assert result is True
    assert requests_mock.call_count == 2


def test_fetch_does_not_send_if_test_set(direqt, requests_mock):
    """Honors the test flag from the server

    Verify that when the Direqt server returns 'isTest', no message
    is sent via RBM.

    :param requests_mock.Mocker requests_mock
    :param direqt.DireqtClient direqt
    """
    rbm_message = {
        'contentMessage': {
            'text': 'Hello, world!'
        }
    }

    direqt_response = {
        'payload': json.dumps(rbm_message),
        'isTest': True
    }

    def verify_rbm_request(request):
        """:type request: requests.models.PreparedRequest"""
        assert False    # should not have been called!

    requests_mock.post(EXPECTED_FETCH_URL, json=direqt_response)  # direqt/fetch
    requests_mock.post(EXPECTED_AGENT_MESSAGE_URL, json={},
                       additional_matcher=verify_rbm_request)

    result = direqt.fetch('test-ad-unit', TEST_MSISDN)

    assert result
    assert requests_mock.call_count == 1    # just the direqt fetch, not the RBM


def test_fetch_media_upload(direqt, requests_mock):
    """Handles a media upload.

    When the fetch response includes a media file, we should upload it.

    :param requests_mock.Mocker requests_mock
    :param direqt.DireqtClient direqt
    """

    file_url = 'http://www.example.com/somefile.jpg'
    thumbnail_url = 'http://www.example.com/somefile-tn.jpg'
    content_description = 'This is a description.'

    expected_rbm_message = {
        'fileUrl': file_url,
        'thumbnailUrl': thumbnail_url,
        'contentDescription': content_description
    }

    # N.B. Currently, 'payload' is a string, while 'media' is JSON...
    direqt_response = {
        'media': [
            {'file_url': file_url, 'thumbnail_url': thumbnail_url, 'content_description': content_description},
        ]
    }

    def verify_rbm_request(request):
        """:type request: requests.models.PreparedRequest"""
        assert request.body == json.dumps(expected_rbm_message)
        return True

    requests_mock.post(EXPECTED_FETCH_URL, json=direqt_response)  # direqt/fetch
    requests_mock.post(EXPECTED_FILE_MESSAGE_URL, json=create_mock_rbm_response(expected_rbm_message),
                       additional_matcher=verify_rbm_request)

    rbm_called = False
    result = direqt.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 2


def test_fetch_provides_proper_sms_parameters(direqt_sms, requests_mock):
    """SDK provides expected default parameters fetch endpoint.
    :param direqt.DireqtClient direqt: default Direqt test client
    """

    def verify_request(request):
        """:type request: requests.models.PreparedRequest"""
        qs = parse_qs(request.url)

        assert request.body is None
        assert qs['key'] == TEST_API_KEY
        assert qs['format'] == 'SMS'
        assert qs['adUnit'] == 'test-ad-unit'
        assert qs['subscriber'] == TEST_MSISDN_MD5
        assert qs['targeting'] == '{}'

        return True

    requests_mock.post(EXPECTED_FETCH_URL, json={}, additional_matcher=verify_request)

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert result is False


def test_fetch_uses_supplied_subscriber_id(direqt_sms, requests_mock):
    """SDK provides expected default parameters fetch endpoint.
    :param direqt.DireqtClient direqt: default Direqt test client
    """

    def verify_request(request):
        """:type request: requests.models.PreparedRequest"""
        qs = parse_qs(request.url)
        assert qs['subscriber'] == TEST_SUBSCRIBER_ID
        return True

    requests_mock.post(EXPECTED_FETCH_URL, json={}, additional_matcher=verify_request)

    result = direqt_sms.fetch('test-ad-unit', subscriber_id=TEST_SUBSCRIBER_ID)

    assert requests_mock.call_count == 1
    assert result is False


def test_fetch_uses_supplied_subscriber_id_when_msisdn_provided(direqt_sms, requests_mock):
    """SDK provides expected default parameters fetch endpoint.
    :param direqt.DireqtClient direqt: default Direqt test client
    """

    def verify_request(request):
        """:type request: requests.models.PreparedRequest"""
        qs = parse_qs(request.url)
        assert qs['subscriber'] == TEST_SUBSCRIBER_ID
        return True

    requests_mock.post(EXPECTED_FETCH_URL, json={}, additional_matcher=verify_request)

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN, subscriber_id=TEST_SUBSCRIBER_ID)

    assert requests_mock.call_count == 1
    assert result is False


def test_fetch_returns_proper_sms_response(direqt_sms, requests_mock):
    """fetch() returns proper response object for SMS.
    :param direqt.DireqtClient direqt_sms: default Direqt test client configured for SMS
    """

    def verify_request(request):
        """:type request: requests.models.PreparedRequest"""
        qs = parse_qs(request.url)

        assert request.body is None
        assert qs['key'] == TEST_API_KEY
        assert qs['format'] == 'SMS'
        assert qs['adUnit'] == 'test-ad-unit'
        assert qs['targeting'] == '{}'

        return True

    requests_mock.post(EXPECTED_FETCH_URL, json={'payload': 'This is test payload', 'messageId': 'test-message-id'}, additional_matcher=verify_request)

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert result is not False
    assert result['payload'] == 'This is test payload'
    assert result['message_id'] == 'test-message-id'


def test_fetch_returns_proper_sms_response_with_custom_properties(direqt_sms, requests_mock):
    """fetch() returns proper response object for SMS.
    :param direqt.DireqtClient direqt_sms: default Direqt test client configured for SMS
    """
    response_json = {
        'payload': 'This is test payload',
        'messageId': 'test-message-id',
        'properties': {
            'foo': 'foo-value',
            'bar': 'bar-value',
            'fff': 42
        }
    }

    requests_mock.post(EXPECTED_FETCH_URL, json=response_json)

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert 'properties' in result
    properties = result['properties']
    assert properties['foo'] == 'foo-value'
    assert properties['bar'] == 'bar-value'
    assert properties['fff'] == 42


def test_fetch_sets_is_test_flag_to_false_by_default(direqt_sms, requests_mock):
    """fetch() sets 'is_test' property to false by default
    :param direqt.DireqtClient direqt_sms: default Direqt test client configured for SMS
    """
    response_json = {
        'payload': 'This is test payload',
        'messageId': 'test-message-id',
        'properties': {
            'foo': 'foo-value',
            'bar': 'bar-value',
            'fff': 42
        }
    }

    requests_mock.post(EXPECTED_FETCH_URL, json=response_json)

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert 'is_test' in result
    assert not result['is_test']


def test_fetch_sets_is_test_flag_to_false_for_false_value(direqt_sms, requests_mock):
    """fetch() sets 'is_test' property to false by default
    :param direqt.DireqtClient direqt_sms: default Direqt test client configured for SMS
    """
    response_json = {
        'payload': 'This is test payload',
        'messageId': 'test-message-id',
        'properties': {
            'foo': 'foo-value',
            'bar': 'bar-value',
            'fff': 42
        },
        'isTest': False
    }

    requests_mock.post(EXPECTED_FETCH_URL, json=response_json)

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert 'is_test' in result
    assert not result['is_test']


def test_fetch_sets_is_test_flag_to_true(direqt_sms, requests_mock):
    """fetch() sets 'is_test' property to false by default
    :param direqt.DireqtClient direqt_sms: default Direqt test client configured for SMS
    """
    response_json = {
        'payload': 'This is test payload',
        'messageId': 'test-message-id',
        'properties': {
            'foo': 'foo-value',
            'bar': 'bar-value',
            'fff': 42
        },
        'isTest': True
    }

    requests_mock.post(EXPECTED_FETCH_URL, json=response_json)

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert 'is_test' in result
    assert result['is_test']


def test_fetch_does_not_leak_msisdn(direqt, requests_mock):
    """Msisdn supplied to fetch should never appear in a server request.
    :param direqt.DireqtClient direqt: default Direqt test client
    """

    def verify_request(request):
        """:type request: requests.models.PreparedRequest"""
        qs = parse_qs(request.url)

        assert request.body is None
        for k in qs:
            assert TEST_MSISDN not in qs[k]

        return True

    requests_mock.post(EXPECTED_FETCH_URL, json={}, additional_matcher=verify_request)

    result = direqt.fetch('test-ad-unit', TEST_MSISDN)

    assert requests_mock.call_count == 1
    assert result is False


def test_fetch_invokes_custom_url_shortener(direqt_sms, requests_mock):
    """fetch() invokes custom shortener for SMS url, and requests longUrl.
    :param direqt.DireqtClient direqt_sms: default Direqt test client configured for SMS
    """

    def verify_request(request):
        """:type request: requests.models.PreparedRequest"""
        qs = parse_qs(request.url)
        assert qs['longUrl'] == 'true'
        return True

    requests_mock.post(EXPECTED_FETCH_URL, json={
        'payload': 'This is test payload http://www.example.com/long/url/here',
        'messageId': 'test-message-id'}, additional_matcher=verify_request)

    def custom_url_shortener(url):
        return 'https://ex.co/short/url'

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN, url_shortener=custom_url_shortener)

    assert requests_mock.call_count == 1
    assert result is not False
    assert result['payload'] == 'This is test payload https://ex.co/short/url'
    assert result['message_id'] == 'test-message-id'


def test_fetch_with_url_shortener_but_no_returned_link(direqt_sms, requests_mock):
    """fetch() behaves as expected when custom shortener specified and no link in returned payload.
    :param direqt.DireqtClient direqt_sms: default Direqt test client configured for SMS
    """
    requests_mock.post(EXPECTED_FETCH_URL, json={
        'payload': 'This payload has no link',
        'messageId': 'test-message-id'})

    def custom_url_shortener(url):
        return 'https://ex.co/short/url'

    result = direqt_sms.fetch('test-ad-unit', TEST_MSISDN, url_shortener=custom_url_shortener)

    assert requests_mock.call_count == 1
    assert result is not False
    assert result['payload'] == 'This payload has no link'




"""Client library for Direqt Ads API."""
import json
import logging
import uuid
import hashlib
import requests
import re

from util import detect_gae
from . import rbm

logger = logging.getLogger(__name__)

_DEFAULT_ENDPOINT = 'https://ads.direqt.io'
_DEFAULT_USER_AGENT = 'unknown'

_AD_FORMAT_RBM10 = 'RBMV1.0'
_AD_FORMAT_SMS = 'SMS'
_DEFAULT_AD_FORMAT = _AD_FORMAT_RBM10
_VALID_AD_FORMATS = [_AD_FORMAT_RBM10, _AD_FORMAT_SMS]

_DEFAULT_RBM_ENDPOINT = 'https://rcsbusinessmessaging.googleapis.com'


class DireqtClient(object):
    """Client for the Direqt ad server."""

    def __init__(self, api_key, ad_format=_DEFAULT_AD_FORMAT, user_agent=_DEFAULT_USER_AGENT, **kwargs):
        """Initializes a DireqtClient.

        Args:
            api_key: A string containing your Direqt API key.
            format: One of 'RBMV1.0' or 'SMS'
            user_agent: An arbitrary string containing only ASCII characters
                that will be used to identify your application. If not set,
                defaults to "unknown".
            **kwargs: Optional keyword arguments.

        Keyword Arguments:
            None defined.
        """
        super(DireqtClient, self).__init__()

        if ad_format not in _VALID_AD_FORMATS:
            raise NotImplementedError(
                'The provided ad format "%s" is invalid. Accepted formats are: %s'
                % (format, _VALID_AD_FORMATS))

        if detect_gae():
            import requests_toolbelt.adapters.appengine
            requests_toolbelt.adapters.appengine.monkeypatch()

        self.api_key = api_key
        self.fmt = ad_format
        self.user_agent = user_agent
        self.endpoint = kwargs.get('endpoint', _DEFAULT_ENDPOINT)               # private
        self.http = kwargs.get('http', requests)                                # private
        self.rbm_endpoint = kwargs.get('rbm_endpoint', _DEFAULT_RBM_ENDPOINT)   # private
        self.rbm_http = kwargs.get('rbm_http', rbm.http)

    def fetch(self, ad_unit, msisdn=None, subscriber_id=None, targeting=None, url_shortener=None):
        """Fetch an ad from the Direqt Ads API.

        If no ad is available, this method returns False.

        If the ad_format specified in the constructor was "RBMV1.0" this method
        will handle the sending of any messages necessary to deliver the
        advertisement.

        If the ad_format is "SMS", this method will return an object containing
        the 'message_id' and 'payload' of the message that should be sent to the
        subscriber using the application's messaging platform.

        Args:
            ad_unit: An ad unit code configured in your project.
            subscriber_id: A string identifying the current user. This identifier
                is used to track interactions with a given user, but is not
                otherwise interpreted.
            msisdn: The MSISDN of the user. This argument is only required when
                using RBM, and is ignored for the case of SMS.
            targeting: An array of tuples specifying name/value pairs for targeting
            url_shortener: A custom function to use for shortening marketing urls
                before embedding in messages. If specified, the default Direqt
                url shortener will not be used, and Direqt will not automatically
                truncate SMS messages to 160 characters.
        """
        targeting = targeting or {}

        if not msisdn and self.fmt != _AD_FORMAT_SMS:
            logger.debug("MSISDN is required for RBM message operations.")
            return False

        if not subscriber_id and msisdn:
            subscriber_id = hashlib.md5(self.api_key + msisdn).hexdigest()

        if not subscriber_id and not msisdn:
            logger.debug("Must specify at least one of (subscriber_id, msisdn).")
            return False

        params = {
            'key': self.api_key,
            'format': self.fmt,
            'adUnit': ad_unit,
            'subscriber': subscriber_id,
            'targeting': json.dumps(targeting)
        }

        if url_shortener:
            params['longUrl'] = 'true'

        url = self.endpoint + "/fetch"
        response = self.http.post(url, params=params, data={})
        if response.status_code == 200:
            body = response.json()
            return self._process_fetch_result(msisdn, body, url_shortener)
        else:
            try:
                errormsg = "Direqt fetch error: {0}".format(json.dumps(response.json()))
            except:
                errormsg = "Direqt fetch error: status_code={0} reason={1}".format(response.status_code, response.reason)

            logger.debug(errormsg)

        return False

    def notify_suggested_reply(self, msisdn, postback_data):
        """Notify of a suggested reply from a user.

        This method should be called whenever your agent is notified that a user
        has responded to a suggestion from an earlier message.

        Suggested replies are only relevant for ad_format "RBMV1.0".

        If the suggested reply was handled by Direqt, this method will return True.
        Otherwise, returns False.
        :type postback_data: str
        """
        if not postback_data or not postback_data.startswith('@direqt'):
            return False

        params = {
            'key': self.api_key,
            'format': self.fmt,
            'postbackData': postback_data
        }

        url = self.endpoint + "/interact"
        response = self.http.post(url, params=params, data={})
        if response.status_code == 200:
            body = response.json()
            self._process_fetch_result(msisdn, body)    # return code ignored

        return True

    def track_custom_event(self, message_id, event_name, event_data):
        """Notify that a custom event has occurred.

        This method can be used to record custom events associated with the
        delivery of or interaction with a message that was received by fetch.
        For example, delivery and conversion notifications can be recorded with
        this method, allowing them to be visible in reports generated by the
        Direqt console.
        """
        params = {
            'key': self.api_key
        }
        data = {
            'messageId': message_id,
            'eventName': event_name,
            'eventData': event_data,
        }
        url = self.endpoint + "/event"
        response = self.http.post(url, params=params, json=data)
        if response.status_code == 200:
            body = response.json()
        else:
            body = None

        return body

    def _process_fetch_result(self, msisdn, body, url_shortener=None):
        if self.fmt == _AD_FORMAT_SMS:
            payload = body['payload'] if 'payload' in body else None
            message_id = body['messageId'] if 'messageId' in body else None
            properties = body['properties'] if 'properties' in body else {}
            is_test = body['isTest'] if 'isTest' in body else False
            if not payload or not message_id:
                return False

            if url_shortener:
                match = re.search("(?P<url>https?://[^\s]+)", payload)
                url = match.group("url") if match else None
                if url:
                    new_url = url_shortener(url)
                    payload = payload.replace(url, new_url)

            response = {
                'payload': payload,
                'message_id': message_id,
                'properties': properties,
                'is_test': is_test
            }

            return response

        media = body['media'] if 'media' in body else []
        payload = body['payload'] if 'payload' in body else None
        properties = body['properties'] if 'properties' in body else {}
        is_test = body['isTest'] if 'isTest' in body else False

        if not media and not payload:
            logger.debug("Unexpected response from Direqt fetch.")
            return False

        logger.info("Received payload: " + (payload or '<none>'))

        for m in media:
            uri = self._upload_media(m['file_url'],
                                     thumbnail_url=m['thumbnail_url'],
                                     content_description=m['content_description'])
            if not uri:
                logger.error('Failed to upload media file.')
                return False

            if payload:
                payload = payload.replace(m['file_url'], uri)

        if is_test:
            return True

        return self._deliver_payload(msisdn, uuid.uuid4(), payload)

    def _deliver_payload(self, msisdn, message_id, payload):
        url = self.rbm_endpoint + "/v1/phones/{0}/agentMessages?messageId={1}".format(msisdn, message_id)

        logger.info("Posting agent message {0} for {1}".format(message_id, msisdn))
        logger.info("  url = " + url)
        logger.info("  payload = " + payload)

        response = self.rbm_http.post(url, data=payload, headers={'Content-Type': 'application/json'})

        logger.info("Received response from " + url)
        logger.info("  status_code: " + str(response.status_code))
        logger.info("  response: " + json.dumps(response.json()))

        success = response.status_code == 200
        if not success:
            logger.error("Post of agent message failed!")

        return success

    def _upload_media(self, file_url, thumbnail_url=None, content_description=None):
        url = self.rbm_endpoint + "/v1/files"
        request = {'fileUrl': file_url}
        if thumbnail_url is not None:
            request['thumbnailUrl'] = thumbnail_url
        if content_description is not None:
            request['contentDescription'] = content_description

        logger.info("Uploading media file")
        logger.info("  url = " + url)
        logger.info("  fileUrl = " + file_url)
        logger.info("  thumbnailUrl = " + thumbnail_url)
        logger.info("  contentDescription = " + content_description)

        response = self.rbm_http.post(url, json=request)

        logger.info("Received response from " + url)
        logger.info("  status_code: " + str(response.status_code))
        logger.info("  response: " + json.dumps(response.json()))

        success = response.status_code == 200
        if not success:
            logger.error("Post of file upload failed!")
            return None

        response_json = response.json()
        if not response_json or 'name' not in response_json:
            logger.error("Unexpected response from file upload.")
            return None

        return response_json['name']



import direqt

MOCK_DIREQT_API_KEY = 'mock-api-key'


def test_constructor_defaults():
    d = direqt.DireqtClient(api_key=MOCK_DIREQT_API_KEY)

    assert d.fmt == "RBMV1.0"
    assert d.endpoint == 'https://ads.direqt.io'
    assert d.rbm_endpoint == 'https://rcsbusinessmessaging.googleapis.com'



try:
    import urllib.parse
except ImportError:
    import urlparse

from conftest import TEST_API_KEY, EXPECTED_EVENT_URL


def parse_qs(url):
    """Return URL parameters as a dictionary of name/value pairs."""
    return dict(urlparse.parse_qsl(urlparse.urlsplit(url).query))

def test_track_custom_event_provides_proper_parameters(direqt, requests_mock):
    """SDK provides expected parameters for custom event endpoint.
    :param direqt.DireqtClient direqt: default Direqt test client
    """
    mock_response = {
        'messageId': 'xx',
        'eventName': 'xx',
        'eventData': { 'foo': 'bar' }
    }
    adapter = requests_mock.post(EXPECTED_EVENT_URL, json=mock_response)

    custom_event_data = {
        'thing_one': 'foo',
        'thing_two': 'bar'
    }
    result = direqt.track_custom_event('test-message-id', 'custom-event-name', custom_event_data)

    assert adapter.call_count == 1
    qs = parse_qs(adapter.last_request.url)
    assert qs['key'] == TEST_API_KEY

    request_body = adapter.last_request.json()
    assert request_body['messageId'] == 'test-message-id'
    assert request_body['eventName'] == 'custom-event-name'
    assert request_body['eventData']['thing_one'] == 'foo'
    assert request_body['eventData']['thing_two'] == 'bar'

    assert result == mock_response




"""Utility Functions."""
import os

__all__ = [
    'detect_gae',
]


def detect_gae():
    """Determine whether or not we're running on GAE.

    The SERVER_SOFTWARE variable starts with 'Development/' when running via
    dev_appserver.py.

    Returns:
      True iff we're running on GAE.
    """
    server_software = os.environ.get('SERVER_SOFTWARE', '')
    return (server_software.startswith('Development/') or
            server_software.startswith('Google App Engine/'))



import google.auth
from google.auth.transport.requests import AuthorizedSession

credentials, project = google.auth.default(
    'https://www.googleapis.com/auth/rcsbusinessmessaging'
)

http = AuthorizedSession(credentials)  # OAuth wrapper for requests



from setuptools import setup

setup(name='direqt',
      version='0.2.3',
      classifiers=[
            'Development Status :: 3 - Alpha',
            'Programming Language :: Python :: 2'
      ],
      description='Client library for Direqt Ads API',
      url='https://github.com/direqt/direqt-sdk-python',
      author='Myk Willis',
      author_email='myk@direqt.io',
      license='Apache License 2.0',
      packages=['direqt'],
      install_requires=[
            'google-auth',
            'requests',
            'requests_toolbelt'
      ],
      platforms='any',
      setup_requires=[
            'pytest-runner'
      ],
      tests_requires=[
            'pytest',
            'requests-mock'
      ],
      zip_safe=False)
